.tree-view-item-value-string
{
color:green;
}

.tree-view-item-value-number
{
color:blue;
}
.tree-view-item-value-boolean
{
color:red;
}

.tree-view-item-value-null
{
color:red;
}

.tree-view-item-value-function
{
color:orange;
}

