<template>
    <div>
        <h2>Developer Dashboard</h2>
        <p class="lead">Apitax combines the power of Scriptax and Commandtax into a quick and easy to use Python package
            to facillitate powerful Restful API Request Scripting.</p>
        <p class="lead">For more help, please see the Apitax <a
                href="https://github.com/ShawnClake/Apitax/blob/master/README.md">documentation</a>.</p>
    </div>
</template>
  
  