# Generated from D:/Programming/Projects/Apitax/apitax/grammar/src\Ah210.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .Ah210Parser import Ah210Parser
else:
    from Ah210Parser import Ah210Parser

# This class defines a complete listener for a parse tree produced by Ah210Parser.
class Ah210Listener(ParseTreeListener):

    # Enter a parse tree produced by Ah210Parser#prog.
    def enterProg(self, ctx:Ah210Parser.ProgContext):
        pass

    # Exit a parse tree produced by Ah210Parser#prog.
    def exitProg(self, ctx:Ah210Parser.ProgContext):
        pass


    # Enter a parse tree produced by Ah210Parser#statements.
    def enterStatements(self, ctx:Ah210Parser.StatementsContext):
        pass

    # Exit a parse tree produced by Ah210Parser#statements.
    def exitStatements(self, ctx:Ah210Parser.StatementsContext):
        pass


    # Enter a parse tree produced by Ah210Parser#statement.
    def enterStatement(self, ctx:Ah210Parser.StatementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#statement.
    def exitStatement(self, ctx:Ah210Parser.StatementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#terminated.
    def enterTerminated(self, ctx:Ah210Parser.TerminatedContext):
        pass

    # Exit a parse tree produced by Ah210Parser#terminated.
    def exitTerminated(self, ctx:Ah210Parser.TerminatedContext):
        pass


    # Enter a parse tree produced by Ah210Parser#non_terminated.
    def enterNon_terminated(self, ctx:Ah210Parser.Non_terminatedContext):
        pass

    # Exit a parse tree produced by Ah210Parser#non_terminated.
    def exitNon_terminated(self, ctx:Ah210Parser.Non_terminatedContext):
        pass


    # Enter a parse tree produced by Ah210Parser#executers.
    def enterExecuters(self, ctx:Ah210Parser.ExecutersContext):
        pass

    # Exit a parse tree produced by Ah210Parser#executers.
    def exitExecuters(self, ctx:Ah210Parser.ExecutersContext):
        pass


    # Enter a parse tree produced by Ah210Parser#expr.
    def enterExpr(self, ctx:Ah210Parser.ExprContext):
        pass

    # Exit a parse tree produced by Ah210Parser#expr.
    def exitExpr(self, ctx:Ah210Parser.ExprContext):
        pass


    # Enter a parse tree produced by Ah210Parser#assignment.
    def enterAssignment(self, ctx:Ah210Parser.AssignmentContext):
        pass

    # Exit a parse tree produced by Ah210Parser#assignment.
    def exitAssignment(self, ctx:Ah210Parser.AssignmentContext):
        pass


    # Enter a parse tree produced by Ah210Parser#flow.
    def enterFlow(self, ctx:Ah210Parser.FlowContext):
        pass

    # Exit a parse tree produced by Ah210Parser#flow.
    def exitFlow(self, ctx:Ah210Parser.FlowContext):
        pass


    # Enter a parse tree produced by Ah210Parser#if_statement.
    def enterIf_statement(self, ctx:Ah210Parser.If_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#if_statement.
    def exitIf_statement(self, ctx:Ah210Parser.If_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#while_statement.
    def enterWhile_statement(self, ctx:Ah210Parser.While_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#while_statement.
    def exitWhile_statement(self, ctx:Ah210Parser.While_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#for_statement.
    def enterFor_statement(self, ctx:Ah210Parser.For_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#for_statement.
    def exitFor_statement(self, ctx:Ah210Parser.For_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#each_statement.
    def enterEach_statement(self, ctx:Ah210Parser.Each_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#each_statement.
    def exitEach_statement(self, ctx:Ah210Parser.Each_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#condition.
    def enterCondition(self, ctx:Ah210Parser.ConditionContext):
        pass

    # Exit a parse tree produced by Ah210Parser#condition.
    def exitCondition(self, ctx:Ah210Parser.ConditionContext):
        pass


    # Enter a parse tree produced by Ah210Parser#block.
    def enterBlock(self, ctx:Ah210Parser.BlockContext):
        pass

    # Exit a parse tree produced by Ah210Parser#block.
    def exitBlock(self, ctx:Ah210Parser.BlockContext):
        pass


    # Enter a parse tree produced by Ah210Parser#callback.
    def enterCallback(self, ctx:Ah210Parser.CallbackContext):
        pass

    # Exit a parse tree produced by Ah210Parser#callback.
    def exitCallback(self, ctx:Ah210Parser.CallbackContext):
        pass


    # Enter a parse tree produced by Ah210Parser#callback_block.
    def enterCallback_block(self, ctx:Ah210Parser.Callback_blockContext):
        pass

    # Exit a parse tree produced by Ah210Parser#callback_block.
    def exitCallback_block(self, ctx:Ah210Parser.Callback_blockContext):
        pass


    # Enter a parse tree produced by Ah210Parser#optional_parameters_block.
    def enterOptional_parameters_block(self, ctx:Ah210Parser.Optional_parameters_blockContext):
        pass

    # Exit a parse tree produced by Ah210Parser#optional_parameters_block.
    def exitOptional_parameters_block(self, ctx:Ah210Parser.Optional_parameters_blockContext):
        pass


    # Enter a parse tree produced by Ah210Parser#sig_parameter.
    def enterSig_parameter(self, ctx:Ah210Parser.Sig_parameterContext):
        pass

    # Exit a parse tree produced by Ah210Parser#sig_parameter.
    def exitSig_parameter(self, ctx:Ah210Parser.Sig_parameterContext):
        pass


    # Enter a parse tree produced by Ah210Parser#call_parameter.
    def enterCall_parameter(self, ctx:Ah210Parser.Call_parameterContext):
        pass

    # Exit a parse tree produced by Ah210Parser#call_parameter.
    def exitCall_parameter(self, ctx:Ah210Parser.Call_parameterContext):
        pass


    # Enter a parse tree produced by Ah210Parser#optional_parameter.
    def enterOptional_parameter(self, ctx:Ah210Parser.Optional_parameterContext):
        pass

    # Exit a parse tree produced by Ah210Parser#optional_parameter.
    def exitOptional_parameter(self, ctx:Ah210Parser.Optional_parameterContext):
        pass


    # Enter a parse tree produced by Ah210Parser#commandtax.
    def enterCommandtax(self, ctx:Ah210Parser.CommandtaxContext):
        pass

    # Exit a parse tree produced by Ah210Parser#commandtax.
    def exitCommandtax(self, ctx:Ah210Parser.CommandtaxContext):
        pass


    # Enter a parse tree produced by Ah210Parser#execute.
    def enterExecute(self, ctx:Ah210Parser.ExecuteContext):
        pass

    # Exit a parse tree produced by Ah210Parser#execute.
    def exitExecute(self, ctx:Ah210Parser.ExecuteContext):
        pass


    # Enter a parse tree produced by Ah210Parser#async_execute.
    def enterAsync_execute(self, ctx:Ah210Parser.Async_executeContext):
        pass

    # Exit a parse tree produced by Ah210Parser#async_execute.
    def exitAsync_execute(self, ctx:Ah210Parser.Async_executeContext):
        pass


    # Enter a parse tree produced by Ah210Parser#await.
    def enterAwait(self, ctx:Ah210Parser.AwaitContext):
        pass

    # Exit a parse tree produced by Ah210Parser#await.
    def exitAwait(self, ctx:Ah210Parser.AwaitContext):
        pass


    # Enter a parse tree produced by Ah210Parser#labels.
    def enterLabels(self, ctx:Ah210Parser.LabelsContext):
        pass

    # Exit a parse tree produced by Ah210Parser#labels.
    def exitLabels(self, ctx:Ah210Parser.LabelsContext):
        pass


    # Enter a parse tree produced by Ah210Parser#label_comp.
    def enterLabel_comp(self, ctx:Ah210Parser.Label_compContext):
        pass

    # Exit a parse tree produced by Ah210Parser#label_comp.
    def exitLabel_comp(self, ctx:Ah210Parser.Label_compContext):
        pass


    # Enter a parse tree produced by Ah210Parser#params_statement.
    def enterParams_statement(self, ctx:Ah210Parser.Params_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#params_statement.
    def exitParams_statement(self, ctx:Ah210Parser.Params_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#options_statement.
    def enterOptions_statement(self, ctx:Ah210Parser.Options_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#options_statement.
    def exitOptions_statement(self, ctx:Ah210Parser.Options_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#delete_statement.
    def enterDelete_statement(self, ctx:Ah210Parser.Delete_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#delete_statement.
    def exitDelete_statement(self, ctx:Ah210Parser.Delete_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#error_statement.
    def enterError_statement(self, ctx:Ah210Parser.Error_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#error_statement.
    def exitError_statement(self, ctx:Ah210Parser.Error_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#return_statement.
    def enterReturn_statement(self, ctx:Ah210Parser.Return_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#return_statement.
    def exitReturn_statement(self, ctx:Ah210Parser.Return_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#login_statement.
    def enterLogin_statement(self, ctx:Ah210Parser.Login_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#login_statement.
    def exitLogin_statement(self, ctx:Ah210Parser.Login_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#endpoint_statement.
    def enterEndpoint_statement(self, ctx:Ah210Parser.Endpoint_statementContext):
        pass

    # Exit a parse tree produced by Ah210Parser#endpoint_statement.
    def exitEndpoint_statement(self, ctx:Ah210Parser.Endpoint_statementContext):
        pass


    # Enter a parse tree produced by Ah210Parser#scoping.
    def enterScoping(self, ctx:Ah210Parser.ScopingContext):
        pass

    # Exit a parse tree produced by Ah210Parser#scoping.
    def exitScoping(self, ctx:Ah210Parser.ScopingContext):
        pass


    # Enter a parse tree produced by Ah210Parser#name.
    def enterName(self, ctx:Ah210Parser.NameContext):
        pass

    # Exit a parse tree produced by Ah210Parser#name.
    def exitName(self, ctx:Ah210Parser.NameContext):
        pass


    # Enter a parse tree produced by Ah210Parser#exports.
    def enterExports(self, ctx:Ah210Parser.ExportsContext):
        pass

    # Exit a parse tree produced by Ah210Parser#exports.
    def exitExports(self, ctx:Ah210Parser.ExportsContext):
        pass


    # Enter a parse tree produced by Ah210Parser#imports.
    def enterImports(self, ctx:Ah210Parser.ImportsContext):
        pass

    # Exit a parse tree produced by Ah210Parser#imports.
    def exitImports(self, ctx:Ah210Parser.ImportsContext):
        pass


    # Enter a parse tree produced by Ah210Parser#casting.
    def enterCasting(self, ctx:Ah210Parser.CastingContext):
        pass

    # Exit a parse tree produced by Ah210Parser#casting.
    def exitCasting(self, ctx:Ah210Parser.CastingContext):
        pass


    # Enter a parse tree produced by Ah210Parser#auth.
    def enterAuth(self, ctx:Ah210Parser.AuthContext):
        pass

    # Exit a parse tree produced by Ah210Parser#auth.
    def exitAuth(self, ctx:Ah210Parser.AuthContext):
        pass


    # Enter a parse tree produced by Ah210Parser#url.
    def enterUrl(self, ctx:Ah210Parser.UrlContext):
        pass

    # Exit a parse tree produced by Ah210Parser#url.
    def exitUrl(self, ctx:Ah210Parser.UrlContext):
        pass


    # Enter a parse tree produced by Ah210Parser#log.
    def enterLog(self, ctx:Ah210Parser.LogContext):
        pass

    # Exit a parse tree produced by Ah210Parser#log.
    def exitLog(self, ctx:Ah210Parser.LogContext):
        pass


    # Enter a parse tree produced by Ah210Parser#count.
    def enterCount(self, ctx:Ah210Parser.CountContext):
        pass

    # Exit a parse tree produced by Ah210Parser#count.
    def exitCount(self, ctx:Ah210Parser.CountContext):
        pass


    # Enter a parse tree produced by Ah210Parser#inject.
    def enterInject(self, ctx:Ah210Parser.InjectContext):
        pass

    # Exit a parse tree produced by Ah210Parser#inject.
    def exitInject(self, ctx:Ah210Parser.InjectContext):
        pass


    # Enter a parse tree produced by Ah210Parser#atom.
    def enterAtom(self, ctx:Ah210Parser.AtomContext):
        pass

    # Exit a parse tree produced by Ah210Parser#atom.
    def exitAtom(self, ctx:Ah210Parser.AtomContext):
        pass


    # Enter a parse tree produced by Ah210Parser#obj_dict.
    def enterObj_dict(self, ctx:Ah210Parser.Obj_dictContext):
        pass

    # Exit a parse tree produced by Ah210Parser#obj_dict.
    def exitObj_dict(self, ctx:Ah210Parser.Obj_dictContext):
        pass


    # Enter a parse tree produced by Ah210Parser#obj_list.
    def enterObj_list(self, ctx:Ah210Parser.Obj_listContext):
        pass

    # Exit a parse tree produced by Ah210Parser#obj_list.
    def exitObj_list(self, ctx:Ah210Parser.Obj_listContext):
        pass


    # Enter a parse tree produced by Ah210Parser#string.
    def enterString(self, ctx:Ah210Parser.StringContext):
        pass

    # Exit a parse tree produced by Ah210Parser#string.
    def exitString(self, ctx:Ah210Parser.StringContext):
        pass


    # Enter a parse tree produced by Ah210Parser#number.
    def enterNumber(self, ctx:Ah210Parser.NumberContext):
        pass

    # Exit a parse tree produced by Ah210Parser#number.
    def exitNumber(self, ctx:Ah210Parser.NumberContext):
        pass


    # Enter a parse tree produced by Ah210Parser#boolean.
    def enterBoolean(self, ctx:Ah210Parser.BooleanContext):
        pass

    # Exit a parse tree produced by Ah210Parser#boolean.
    def exitBoolean(self, ctx:Ah210Parser.BooleanContext):
        pass


