class State:
    config = None
    options = None
    log = None
    paths = {
        "node": "",
        "config": "",
        "root": "",
        "apitax": "",
        "log": "",
    }
    baseUrl = "/apitax/2/"
